import os
import json

import sqlalchemy as sa

import db_models
import api_schemas


DB_ADDRESS = os.environ['DB_ADDRESS']
DB_PORT = os.environ['DB_PORT']
DB_USER = os.environ['DB_USER']
DB_PASSWORD = os.environ['DB_PASSWORD']

VERSIONS = json.loads(os.environ['DB_INIT_VERSIONS'])

engines = {
    version: sa.create_engine(
        f'mysql+pymysql://{DB_USER}:{DB_PASSWORD}@{DB_ADDRESS}:{DB_PORT}/db_{version}?charset=utf8'
    ) for version in VERSIONS
}

LocalSession = {
    version: sa.orm.sessionmaker(engine) for version, engine in engines.items()
}


for version, engine in engines.items():
    db_models.Base.metadata.create_all(bind=engine, checkfirst=True)


def get_db(version):
    db = LocalSession[version]()

    try:
        yield db
    finally:
        db.close()


def reset_db(preset: api_schemas.Preset):
    version = preset.version

    engine = engines[version]

    db_models.Base.metadata.drop_all(bind=engine, checkfirst=True)
    db_models.Base.metadata.create_all(bind=engine, checkfirst=False)

    with LocalSession[version]() as session:
        session.add_all(
            db_models.Intermediary(id=intm.id, name=intm.name) for intm in preset.intermediaries
        )
        session.commit()
